#!/usr/bin/env python3

import os
import sys
import subprocess
import yaml
import argparse
import shutil
import platform
from pathlib import Path

def find_pac_command():
    """Find the pac command executable, handling Windows-specific cases"""
    # Try common command names
    pac_commands = ['pac']
    
    # On Windows, also try pac.exe and pac.cmd
    if platform.system() == 'Windows':
        pac_commands.extend(['pac.exe', 'pac.cmd'])
    
    for pac_cmd in pac_commands:
        try:
            # Try to run the command to see if it exists
            result = subprocess.run([pac_cmd, '--version'], 
                                  capture_output=True, text=True, encoding='utf-8', errors='replace', timeout=10)
            if result.returncode == 0:
                return pac_cmd
        except (FileNotFoundError, subprocess.TimeoutExpired):
            continue
    
    # If no pac command found, try to provide better error message
    return None

def run_pac_command(args):
    """Run a pac command with proper error handling"""
    pac_cmd = find_pac_command()
    if not pac_cmd:
        print("\nERROR: 'pac' command not found!")
        print("\nThe Microsoft Power Platform CLI is required but not installed or not in PATH.")
        
        # Show different instructions based on OS
        if platform.system() == 'Windows':
            print("\nTo install on Windows:")
            print("1. Download from: https://aka.ms/PowerAppsCLI")
            print("2. Or install via winget: winget install Microsoft.PowerPlatformCLI")
            print("3. Or install via npm: npm install -g @microsoft/powerplatform-cli")
            print("\nIf already installed, try:")
            print("- Restart your terminal/command prompt")
            print("- Check if pac is in your PATH by running: where pac")
        else:
            print("\nTo install:")
            print("- Install via npm: npm install -g @microsoft/powerplatform-cli")
            print("- Or download from: https://aka.ms/PowerAppsCLI")
        
        print("\nAfter installation, restart your terminal and try again.")
        return None
    
    try:
        command = [pac_cmd] + args
        result = subprocess.run(command, capture_output=True, text=True, encoding='utf-8', errors='replace', check=True)
        return result.stdout
    except subprocess.CalledProcessError as e:
        print(f"Error running pac command: {e}")
        if e.stderr:
            print(f"Error details: {e.stderr}")
        return None
    except Exception as e:
        print(f"Unexpected error running pac command: {e}")
        return None

def run_pac_command_with_status(args):
    """Run a pac command and return (success, stdout, stderr)"""
    pac_cmd = find_pac_command()
    if not pac_cmd:
        print("\nERROR: 'pac' command not found!")
        print("\nThe Microsoft Power Platform CLI is required but not installed or not in PATH.")
        
        # Show different instructions based on OS
        if platform.system() == 'Windows':
            print("\nTo install on Windows:")
            print("1. Download from: https://aka.ms/PowerAppsCLI")
            print("2. Or install via winget: winget install Microsoft.PowerPlatformCLI")
            print("3. Or install via npm: npm install -g @microsoft/powerplatform-cli")
            print("\nIf already installed, try:")
            print("- Restart your terminal/command prompt")
            print("- Check if pac is in your PATH by running: where pac")
        else:
            print("\nTo install:")
            print("- Install via npm: npm install -g @microsoft/powerplatform-cli")
            print("- Or download from: https://aka.ms/PowerAppsCLI")
        
        print("\nAfter installation, restart your terminal and try again.")
        return (False, None, None)
    
    try:
        command = [pac_cmd] + args
        result = subprocess.run(command, capture_output=True, text=True, encoding='utf-8', errors='replace')
        return (result.returncode == 0, result.stdout, result.stderr)
    except Exception as e:
        print(f"Unexpected error running pac command: {e}")
        return (False, None, str(e))

def get_auth_list():
    """Get the list of available environments from pac auth list"""
    return run_pac_command(['auth', 'list'])

def get_pages_list():
    """Get the list of available Power Pages sites from pac pages list"""
    return run_pac_command(['pages', 'list'])

def select_site_interactive():
    """Interactive site selection from pac pages list"""
    print("\nGetting available sites from current environment...")
    pages_list = get_pages_list()
    
    if not pages_list:
        print("Failed to get pages list")
        sys.exit(1)
    
    print("\nAvailable sites:")
    print(pages_list)
    
    # Parse the pages list to extract site information
    lines = pages_list.strip().split('\n')
    sites = []
    
    for line in lines:
        # Look for lines that contain site information (with index, website id, and friendly name)
        if '[' in line and ']' in line and len(line.split()) >= 3:
            try:
                # Extract index, website id, and friendly name
                parts = line.strip().split()
                if len(parts) >= 3:
                    index_part = parts[0]  # [1]
                    website_id = parts[1]  # f3d0c1af-e8c9-451a-920d-18c75f80f252
                    friendly_name = ' '.join(parts[2:])  # OREF - oref-dev
                    
                    if index_part.startswith('[') and index_part.endswith(']'):
                        index = int(index_part[1:-1])
                        sites.append((index, website_id, friendly_name))
            except (ValueError, IndexError):
                continue
    
    if not sites:
        print("No sites found in the current environment")
        sys.exit(1)
    
    # Interactive site selection
    while True:
        try:
            print(f"\nEnter the index number of the site to download:")
            choice = input("Site Index: ").strip()
            index = int(choice)
            
            # Find the selected site
            selected_site = None
            for site_index, website_id, friendly_name in sites:
                if site_index == index:
                    selected_site = (website_id, friendly_name)
                    break
            
            if selected_site:
                website_id, friendly_name = selected_site
                print(f"Selected site: {friendly_name} (ID: {website_id})")
                return website_id, friendly_name
            else:
                print(f"Invalid index {index}. Please try again.")
                
        except ValueError:
            print("Please enter a valid number.")
        except KeyboardInterrupt:
            print("\nOperation cancelled.")
            sys.exit(0)

def select_environment_interactive(prompt_message):
    """Interactive environment selection"""
    print(f"\n{prompt_message}")
    auth_list = get_auth_list()
    
    if not auth_list:
        print("Failed to get authentication list")
        sys.exit(1)
    
    print("\nAvailable environments:")
    print(auth_list)
    
    # Parse the auth list to extract environment names and indices
    lines = auth_list.strip().split('\n')
    environments = []
    
    for i, line in enumerate(lines):
        if line.strip() and not line.startswith('Index'):
            environments.append((i, line.strip()))
    
    if not environments:
        print("No environments found")
        sys.exit(1)
    
    environments.append((-1, "Add new environment"))
    
    # Simple input selection (you can enhance this with arrow key navigation later)
    while True:
        try:
            print(f"\nEnter the index number of the environment (or 0 for a new environment):")
            choice = input("Index: ").strip()
            index = int(choice)
            
            if index == 0:
                # Add new environment
                new_env_url = input("Enter the URL for the new environment: ").strip()
                print(f"Creating a new environment at {new_env_url}...")
                success, stdout, stderr = run_pac_command_with_status(['auth', 'create', '--url', new_env_url])
                if success:
                    print("New environment added successfully!")
                    # Re-fetch and display the updated list of environments
                    auth_list = get_auth_list()
                    if not auth_list:
                        print("Failed to get authentication list")
                        sys.exit(1)
                    
                    print("\nUpdated environments:")
                    print(auth_list)
                    continue
                else:
                    print(f"Failed to create new environment: {stderr}")
                    continue

            # Validate the index exists
            success, stdout, stderr = run_pac_command_with_status(['auth', 'select', '--index', str(index)])
            if success:
                print(f"Selected environment at index {index}")
                return index
            else:
                print(f"Invalid index {index}. Please try again.")
        except ValueError:
            print("Please enter a valid number.")
        except KeyboardInterrupt:
            print("\nOperation cancelled.")
            sys.exit(0)

def load_site_config(folder_path):
    """Load site configuration from website.yml in the given folder"""
    website_yml_path = Path(folder_path) / 'website.yml'
    
    if not website_yml_path.exists():
        print(f"Error: website.yml not found in {folder_path}")
        sys.exit(1)
    
    try:
        with open(website_yml_path, 'r') as file:
            data = yaml.safe_load(file)
            site_id = data.get('adx_websiteid')
            if not site_id:
                print(f"Error: adx_websiteid not found in {website_yml_path}")
                sys.exit(1)
            
            # Get the project name from the folder name
            project_name = Path(folder_path).name
            
            return {
                'site_id': site_id,
                'project_name': project_name
            }
    except yaml.YAMLError as e:
        print(f"Error parsing YAML file: {e}")
        sys.exit(1)
    except Exception as e:
        print(f"Error reading file: {e}")
        sys.exit(1)

def run_command(command):
    """Run a shell command and return the result"""
    print(f"Executing: {' '.join(command) if isinstance(command, list) else command}")
    
    if isinstance(command, str):
        result = subprocess.run(command, shell=True, text=True)
    else:
        result = subprocess.run(command, text=True)
    
    return result.returncode

def download_site(web_site_id, download_path="./"):
    """Download site from selected environment"""
    print(f"\nDownloading site with ID: {web_site_id}")
    
    # Use run_pac_command_with_status to get proper error handling
    success, stdout, stderr = run_pac_command_with_status(['pages', 'download', '--webSiteId', web_site_id, '--path', download_path, '-mv', '2', '-o'])
    
    if success:
        if stdout:
            print(stdout)
        return 0
    else:
        if stderr:
            print(f"Download error: {stderr}")
        return 1

def upload_site(upload_path):
    """Upload site to selected environment"""
    print(f"\nUploading site from path: {upload_path}")
    
    # Use run_pac_command_with_status to get proper error handling
    success, stdout, stderr = run_pac_command_with_status(['pages', 'upload', '--path', upload_path, '--modelVersion', '2'])
    
    if success:
        if stdout:
            print(stdout)
        return 0
    else:
        if stderr:
            print(f"Upload error: {stderr}")
        return 1

def transfer_site(download_path=None, upload_path=None):
    """Main function to transfer site between environments"""
    
    # Set default paths if not provided
    if download_path is None:
        download_path = "./"
    
    print(f"Download path: {download_path}")
    
    # Select source environment for download
    select_environment_interactive("Select SOURCE environment to download from:")
    
    # Select site from the current environment
    site_id, site_friendly_name = select_site_interactive()
    
    # Download the site
    if download_site(site_id, download_path) != 0:
        print("Download failed!")
        sys.exit(1)
    
    print("Download completed successfully!")
    
    # Find the downloaded folder
    if upload_path is None:
        download_dir = Path(download_path)
        downloaded_folders = [f for f in download_dir.iterdir() 
                            if f.is_dir() and f.name != '.git' and f.name != '__pycache__']
        
        if len(downloaded_folders) == 1:
            upload_path = str(downloaded_folders[0])
            print(f"Found downloaded folder: {upload_path}")
        elif len(downloaded_folders) > 1:
            print("\nMultiple folders found. Select the downloaded site folder:")
            for i, folder in enumerate(downloaded_folders, 1):
                print(f"[{i}] {folder.name}")
            
            while True:
                try:
                    choice = input("\nEnter folder number: ").strip()
                    folder_index = int(choice) - 1
                    if 0 <= folder_index < len(downloaded_folders):
                        upload_path = str(downloaded_folders[folder_index])
                        print(f"Selected folder: {upload_path}")
                        break
                    else:
                        print("Invalid selection. Please try again.")
                except ValueError:
                    print("Please enter a valid number.")
                except KeyboardInterrupt:
                    print("\nOperation cancelled.")
                    sys.exit(0)
        else:
            print("No downloaded folder found!")
            sys.exit(1)
    
    print(f"Upload path: {upload_path}")
    
    # Select target environment for upload
    select_environment_interactive("Select TARGET environment to upload to:")
    
    # Upload the site
    if upload_site(upload_path) != 0:
        print("Upload failed!")
        sys.exit(1)
    
    print("Upload completed successfully!")
    print("Site transfer completed!\n")

    # Delete the downloaded folder
    try:
        if upload_path:
            print(f"Deleting downloaded folder: {upload_path}")
            shutil.rmtree(upload_path)
            print("Folder deleted successfully!")
    except Exception as e:
        print(f"Failed to delete folder: {e}")

def main():
    parser = argparse.ArgumentParser(
        description='Zooz CLI - Transfer Power Pages sites between environments',
        prog='zooz'
    )
    
    subparsers = parser.add_subparsers(dest='command', help='Available commands')
    
    # transfer-site command
    transfer_parser = subparsers.add_parser(
        'transfer-site',
        help='Transfer a site between environments'
    )
    transfer_parser.add_argument(
        '--download-path',
        default='./',
        help='Path where to download the site (default: ./)'
    )
    transfer_parser.add_argument(
        '--upload-path',
        help='Path from where to upload the site (default: auto-detected from downloaded folder)'
    )
    
    args = parser.parse_args()
    
    if not args.command:
        parser.print_help()
        sys.exit(1)
    
    if args.command == 'transfer-site':
        transfer_site(args.download_path, args.upload_path)

if __name__ == '__main__':
    main()
